import numpy as np
from src.plot import plot_xy
import time
from src.TiseSolverFgh1D import TiseSolverFgh1D
import os
from pathlib import Path

if __name__ == "__main__":
    # - Input parameters
    # -- Position
    xa = -10.0  # Position x = [xa, xb]
    xb = 10.0
    nx = 2**6  # Number of basis for FGH method
    # -- How many eigenvalues and eigenvectors you want from the smallest eigenvalue? (num_eig < nx*ny)
    num_eig = nx // 4
    # -- Eigenfunction abscissa
    x = np.linspace(xa, xb, 4 * nx, endpoint=False, dtype=np.float64)
    # -- Output folder
    output_folder_path = "./out/"

    # -- Preparetion
    folder_path = Path(output_folder_path)  # Make folder
    folder_path.mkdir(exist_ok=True, parents=True)

    # =====================
    #         TISE
    # =====================

    # -- Solve TISE using Fourier-Grid Hamiltonian method
    solver = TiseSolverFgh1D(xa, xb, nx, num_eig)
    start_time = time.time()  # Record the start time
    eigenvalues = solver.solve()
    eigenfunctions = solver.eigenfunctions_scaling(x)
    Vx = solver.V(x)
    end_time = time.time()  # Record the end time
    print(f"  Elapsed time TISE: {end_time - start_time:.6f} seconds")

    # -- Eigenvalues output
    output_csv_name = os.path.join(output_folder_path, "eigenvalues.csv")
    with open(output_csv_name, "w") as f:
        for idx, value in enumerate(eigenvalues):
            f.write(f"{idx},{value}\n")

    # -- Eigenfunctions output
    x_dict = {}
    y_dict = {}
    data_optional = {}
    lv = "$V(x)$"
    x_dict[lv] = np.array(x)
    y_dict[lv] = np.array(Vx)
    data_optional[lv] = {"fmt": "-", "lw": 1, "lc": "black"}
    for n in range(5):
        lv = f"Idx {n:>2d}"
        x_dict[lv] = np.array(x)
        y_dict[lv] = eigenfunctions[n, :].real + eigenvalues[n].real
        data_optional[lv] = {"fmt": "-", "lw": 1, "lc": f"C{n}"}
    output_image_name = os.path.join(output_folder_path, "eigenfunctions.png")
    plot_xy(
        x_dict,
        y_dict,
        data_optional=data_optional,
        show_gui=True,
        output_image_name=output_image_name,
    )
